<?php

namespace Teknisa\PLA\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO\Response\Method;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;
use \Setforms2\Shared\ORM\DateTime;
use \Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use \Zeedhi\Framework\DB\StoredProcedure\Param;
use Exception;

class AuthController {

	protected $requisicaoServico;
	protected $entityManager;
	protected $novoCodigo;
	protected $environment;
	protected $dataSourceManager;
	protected $crudService;

	public function __construct($entityManager, $novoCodigo, $dataSourceManager, $environment, $crudService) {
		$this->entityManager     = $entityManager;
		$this->novoCodigo        = $novoCodigo;
		$this->dataSourceManager = $dataSourceManager;
		$this->environment       = $environment;
		$this->crudService       = $crudService;
	}
	/*
	 * CDOPERADOR
	 * AUTENTICACAO =  (ALIAS_NAME => 'ALIAS', COLUMN_NAME => 'COLUMN NAME')
	*/
	public function isValidAccess(DTO\Request $request, DTO\Response $response){
		$row = $request->getRow();
		$cdGrupOper = null;
		$perfAcess = null;
		$permission = true;
		
		$perfAcess = $this->getPerfilAcesso($row['AUTENTICACAO']['ALIAS_NAME']);
		if(!$perfAcess){
			if($row['AUTENTICACAO']['COLUMN_NAME']){
				$cdGrupOper = $this->getGrupOperParamGeral($row['AUTENTICACAO']['COLUMN_NAME']);
			 } else{
				$cdGrupOper = $this->getGrupOperAutgrupop($row['AUTENTICACAO']['ALIAS_NAME']);
			 }
		}
	
		if(!$cdGrupOper && !$perfAcess){
			$permission = true;
		}else{
			if($perfAcess){
				$result = $this->validPerfilAcesso($perfAcess, $row['CDOPERADOR']);
			}else{
				$result = $this->validGrupOper($cdGrupOper, $row['CDOPERADOR']);
			}
			$permission = !empty($result);
		}
		
		$response->addDataSet(new DataSet ('permission', array('isValid' => $permission)));
	}
    
    public function validGrupOper($cdGrupOper, $cdOperador){
        $dataSourceName = 'opergrupop';
		$conditions = array(
			array(
				'columnName' => 'CDGRUPOPER',
				'operator' => '=',
				'value' => $cdGrupOper
			),
			array(
				'columnName' => 'CDOPERADOR',
				'operator' => '=',
				'value' => $cdOperador
			)
		);
		$result = $this->crudService->getFilteredDataSet($dataSourceName, $conditions)->getRows();
		return $result;
    }
    
    public function getGrupOperParamGeral($columnName){
        $dataSourceName = 'paramgeral';
		$conditions = array();
		$result = $this->crudService->getFilteredDataSet($dataSourceName, $conditions)->getRows();
		
		if(!empty($result)){
		    return $result[0][$columnName];
		}else{
		    return null;
		}
    }
    
    public function getGrupOperAutgrupop($idautgrupop){
        $dataSourceName = 'autgrupop';
		$conditions = array(
			array(
				'columnName' => 'IDAUTGRUPOP',
				'operator' => 'LIKE',
				'value' => $idautgrupop
			)
		);
		$result = $this->crudService->getFilteredDataSet($dataSourceName, $conditions)->getRows();
		
		if(!empty($result)){
		    return $result[0]['CDGRUPOPER'];
		}else{
		    return null;
		}
    }
    
    public function authOperator(DTO\Request $request, DTO\Response $response) {
		try{

			$row = $request->getRow();

			$operator = $this->findCdOperator($row['EMAIL']);
			if(!empty($row['EMAIL'])){
				$senha = $row['SENHA'];
				$this->validatePassword($operator, $row['SENHA']);
			}

			$row['CDOPERADOR'] = $operator;
			$this->isValidAccess($request, $response);
		    
		} catch (\Exception $e) {
			$response->setError(new Error($e->getMessage(), 500));
		}
	}
	
	public function findCdOperator($email) {
		$dataSourceName = 'operador';
		$conditions = array(
			array(
				'columnName' => 'DSEMAILOPER',
				'operator' => 'LIKE',
				'value' => $email
			)
		);
		$dataSet = $this->crudService->getFilteredDataSet($dataSourceName, $conditions)->getRows();
		if(empty($dataSet)){
			throw new \Exception("Operador não cadastrado", 1);
		} else {
			return $dataSet[0]['CDOPERADOR'];
		}
	}
	
	public function validatePassword($operator, $password) {
		$dataSourceName = 'operador';
		$conditions = array(
			array(
				'columnName' => 'CDOPERADOR',
				'operator' => '=',
				"value" => $operator
			)
		);
		$dataSet = $this->crudService->getFilteredDataSet($dataSourceName, $conditions)->getRows();
		if(md5($password) != $dataSet[0]['CDSENHAOPERWEB']){
			throw new \Exception("Senha inválida", 1);
		}
	}
	
    public function validPerfilAcesso($nrperfilacesso, $cdOperador){
        $dataSourceName = 'perfAcessParcOperador';
		$conditions = array(
			array(
				'columnName' => 'NRPERFILACESSO',
				'operator' => '=',
				'value' => $nrperfilacesso
			),
			array(
				'columnName' => 'CDOPERADOR',
				'operator' => '=',
				'value' => $cdOperador
			)
		);
		
		$result = $this->crudService->getFilteredDataSet($dataSourceName, $conditions)->getRows();
		return $result;
    }

    public function getPerfilAcesso($idautperfacesso){
        $dataSourceName = 'autperfilacesso';
		$conditions = array(
			array(
				'columnName' => 'IDAUTPERFACESSO',
				'operator' => 'LIKE',
				'value' => $idautperfacesso
			)
		);
		$result = $this->crudService->getFilteredDataSet($dataSourceName, $conditions)->getRows();
		
		if(!empty($result)){
		    return $result[0]['NRPERFILACESSO'];
		}else{
		    return null;
		}
    }
	
	public function validationDependingGroup($request, $response){
		$row = $request->getRow();
		$cdGrupOper = null;
		$perfAcess = null;
		$permission = true;
		
		$perfAcess = $this->getPerfilAcesso($row['ALIAS_NAME']);
		if(!$perfAcess){
			if($row['COLUMN_NAME']){
				$cdGrupOper = $this->getGrupOperParamGeral($row['COLUMN_NAME']);
			 } else{
				$cdGrupOper = $this->getGrupOperAutgrupop($row['ALIAS_NAME']);
			 }
		}
	
		if(!$cdGrupOper && !$perfAcess){
			$str = 'NOGROUP';
			$response->addDataSet(new DataSet ('permission', array('result' => $str)));
		}else{
			if($perfAcess){
				$result = $this->validPerfilAcesso($perfAcess, $this->environment->getCdOperador());
			}else{
				$result = $this->validGrupOper($cdGrupOper, $this->environment->getCdOperador());
			}
			$str = !empty($result) ? "VALID" : 'NOPERMISSION';
			$response->addDataSet(new DataSet ('permission', array('result' => $str)));
		}
	}
	
	public function returnOperatorInGroup($row){
		$cdGrupOper = null;
		$perfAcess = null;
		$permission = true;
		
		$perfAcess = $this->getPerfilAcesso($row['AUTENTICACAO']['ALIAS_NAME']);
		if(!$perfAcess){
			if($row['AUTENTICACAO']['COLUMN_NAME']){
				$cdGrupOper = $this->getGrupOperParamGeral($row['AUTENTICACAO']['COLUMN_NAME']);
			 } else{
				$cdGrupOper = $this->getGrupOperAutgrupop($row['AUTENTICACAO']['ALIAS_NAME']);
			 }
		}
	
		if(!$cdGrupOper && !$perfAcess){
			$permission = true;
		}else{
			if($perfAcess){
				$result = $this->validPerfilAcesso($perfAcess, $this->environment->getCdOperador());
			}else{
				$result = $this->validGrupOper($cdGrupOper, $this->environment->getCdOperador());
			}
			$permission = !empty($result);
		}
		
		return $permission;
	}

}